#include "ps2.h"

#define BUFFER_SIZE 128
static volatile uint8_t ps2buffer[BUFFER_SIZE];
static volatile uint8_t ps2bufsize = 0;
static uint8_t DataPin;
extern uint8_t portram[0x400];
uint8_t kbloop = 0;

void ps2poll();

void ps2interrupt(void)
{
  static uint8_t bitcount=0;
  static uint8_t incoming=0;
  static uint32_t prev_ms=0;
  uint32_t now_ms;
  uint8_t n, val;

  delayMicroseconds(3);

  val = digitalRead(DataPin);
  now_ms = millis();
  if (now_ms - prev_ms > 250) {
    bitcount = 0;
    incoming = 0;
  }
  prev_ms = now_ms;
  n = bitcount - 1;
  if (n <= 7) {
    incoming |= (val << n);
  }
  bitcount++;
  if (bitcount == 11) {
    if (ps2bufsize < BUFFER_SIZE) {
      ps2buffer[ps2bufsize++] = incoming;
      ps2poll();
    }
    bitcount = 0;
    incoming = 0;
  }
  kbloop = 1;
}

uint8_t ps2_to_xt_1(uint32_t val) {
  uint8_t i;
  for (i=0; i<85; i++) {
    if (ps2_group1[i].make == val) return ps2_group1[i].xt_make;
  }
  return 0;
}

uint8_t ps2_to_xt_2(uint32_t val) {
  uint8_t i;
  for (i=0; i<16; i++) {
    if (ps2_group2[i].xt_make == val) return ps2_group2[i].make;
  }
  return 0;
}

uint32_t ps2getcode() {
  uint32_t retval, i, len;
  if (!ps2bufsize) return 0;
  switch (ps2buffer[0]) {
    case 0xF0:
    case 0xE0:
    case 0xE1:
      len = 2;
      break;
    default:
      len = 1;
      break;
  }
  if (ps2bufsize < len) return 0;
  if (ps2buffer[0] == 0xE0) {
    if (ps2buffer[1] == 0xF0) len = 3;
  }
  if (ps2bufsize < len) return 0;
  retval = 0;

  //for (i=0; i<len; i++) {
  //    Serial.write(8);
  //    Serial.write(ps2buffer[i]);
  //}
  
  //translate code
  if (len == 1) {
    retval = ps2_to_xt_1(ps2buffer[0]);
  }
  if (len == 2) {
    if (ps2buffer[0] == 0xF0) retval = ps2_to_xt_1(ps2buffer[1]) | 0x80;
    if (ps2buffer[0] == 0xE0) retval = ps2_to_xt_2(ps2buffer[1]);
  }
  if (len == 3) {
    if ((ps2buffer[0] == 0xE0) && (ps2buffer[1] == 0xF0)) retval = ps2_to_xt_2(ps2buffer[2]) | 0x80;
  }
  //end translate code
  
  for (i=len; i<BUFFER_SIZE; i++) {
    ps2buffer[i-len] = ps2buffer[i];
  }
  ps2bufsize -= len;
  return retval;
}

void setup_ps2(uint8_t data_pin, uint8_t irq_pin) {
  uint8_t irq_num=255;

  DataPin = data_pin;

  // initialize the pins
#ifdef INPUT_PULLUP
  pinMode(irq_pin, INPUT_PULLUP);
  pinMode(data_pin, INPUT_PULLUP);
#else
  pinMode(irq_pin, INPUT);
  digitalWrite(irq_pin, HIGH);
  pinMode(data_pin, INPUT);
  digitalWrite(data_pin, HIGH);
#endif

  irq_num = digitalPinToInterrupt(irq_pin);
/*#ifdef CORE_INT_EVERY_PIN
  irq_num = irq_pin;

#else
  switch(irq_pin) {
    #ifdef CORE_INT0_PIN
    case CORE_INT0_PIN:
      irq_num = 0;
      break;
    #endif
    #ifdef CORE_INT1_PIN
    case CORE_INT1_PIN:
      irq_num = 1;
      break;
    #endif
    #ifdef CORE_INT2_PIN
    case CORE_INT2_PIN:
      irq_num = 2;
      break;
    #endif
    #ifdef CORE_INT3_PIN
    case CORE_INT3_PIN:
      irq_num = 3;
      break;
    #endif
    #ifdef CORE_INT4_PIN
    case CORE_INT4_PIN:
      irq_num = 4;
      break;
    #endif
    #ifdef CORE_INT5_PIN
    case CORE_INT5_PIN:
      irq_num = 5;
      break;
    #endif
    #ifdef CORE_INT6_PIN
    case CORE_INT6_PIN:
      irq_num = 6;
      break;
    #endif
    #ifdef CORE_INT7_PIN
    case CORE_INT7_PIN:
      irq_num = 7;
      break;
    #endif
    #ifdef CORE_INT8_PIN
    case CORE_INT8_PIN:
      irq_num = 8;
      break;
    #endif
    #ifdef CORE_INT9_PIN
    case CORE_INT9_PIN:
      irq_num = 9;
      break;
    #endif
    #ifdef CORE_INT10_PIN
    case CORE_INT10_PIN:
      irq_num = 10;
      break;
    #endif
    #ifdef CORE_INT11_PIN
    case CORE_INT11_PIN:
      irq_num = 11;
      break;
    #endif
    #ifdef CORE_INT12_PIN
    case CORE_INT12_PIN:
      irq_num = 12;
      break;
    #endif
    #ifdef CORE_INT13_PIN
    case CORE_INT13_PIN:
      irq_num = 13;
      break;
    #endif
    #ifdef CORE_INT14_PIN
    case CORE_INT14_PIN:
      irq_num = 14;
      break;
    #endif
    #ifdef CORE_INT15_PIN
    case CORE_INT15_PIN:
      irq_num = 15;
      break;
    #endif
    #ifdef CORE_INT16_PIN
    case CORE_INT16_PIN:
      irq_num = 16;
      break;
    #endif
    #ifdef CORE_INT17_PIN
    case CORE_INT17_PIN:
      irq_num = 17;
      break;
    #endif
    #ifdef CORE_INT18_PIN
    case CORE_INT18_PIN:
      irq_num = 18;
      break;
    #endif
    #ifdef CORE_INT19_PIN
    case CORE_INT19_PIN:
      irq_num = 19;
      break;
    #endif
    #ifdef CORE_INT20_PIN
    case CORE_INT20_PIN:
      irq_num = 20;
      break;
    #endif
    #ifdef CORE_INT21_PIN
    case CORE_INT21_PIN:
      irq_num = 21;
      break;
    #endif
    #ifdef CORE_INT22_PIN
    case CORE_INT22_PIN:
      irq_num = 22;
      break;
    #endif
    #ifdef CORE_INT23_PIN
    case CORE_INT23_PIN:
      irq_num = 23;
      break;
    #endif
  }
#endif*/

  if (irq_num < 255) {
    attachInterrupt(irq_num, ps2interrupt, FALLING);
  }
}

void ps2poll() {
  uint32_t ps2scancode;
  ps2scancode = ps2getcode();
  if (!ps2scancode) return;
  //Serial.println(ps2scancode, HEX);
  //Serial.write(8);
  //Serial.write(ps2scancode);
  portram[0x60] = ps2scancode;
  portram[0x64] |= 2;
  doirq(1);
}

