#include "emu.h"
#include <SPI.h>
#include <SD.h>

struct i8253_s {
  uint16_t chandata[3];
  uint8_t accessmode[3];
  uint8_t bytetoggle[3];
  uint32_t effectivedata[3];
  float chanfreq[3];
  uint8_t active[3];
  uint16_t counter[3];
};

extern struct i8253_s i8253;

Sd2Card card;

void doirq(uint8_t irqnum);
void write86(uint32_t addr32, uint8_t value);
uint8_t read86(uint32_t addr32);

uint8_t curkey = 0;

extern uint8_t bootdrive, hdcount, updatedscreen;

#ifdef MEGA
uint8_t SPI_RAM_pins[8] = { 4, 5, 6, 7, 0, 0, 0, 0 }; //{ 6, 5, 4, 9, 8, 7, 0, 0 };
#else
uint8_t SPI_RAM_pins[8] = { 25, 26, 27, 28, 29, 30, 31, 32 };
#endif

void pollkb() {
  if (!curkey) {
    if (Serial.available()) {
      curkey = Serial.read();
      doirq(1);
    }
  }
}

void setup() {
  uint8_t i;
  pinMode(13, INPUT);
  pinMode(12, INPUT);
  pinMode(11, INPUT);
  pinMode(10, OUTPUT);
  pinMode(NET_PIN, OUTPUT);
  digitalWrite(NET_PIN, HIGH);

#ifdef MEGA
  DDRG = 0xFF;
  DDRE = 0xFF;
  DDRH = 0xFF;
#endif

  Serial.begin(BAUD_RATE);
  //while(!Serial);
#ifdef USB_DISK
  bootdrive = 0x80;
  insertdisk(0x80);
  hdcount = 1;
#else
  if (!card.init(SPI_HALF_SPEED, SD_PIN)) { //boot to ROM BASIC if no SD card
    bootdrive = 0xFF;
    //Serial.println("SD card init FAILED");
  } else { //else boot to SD as first hard drive
    //Serial.println("SD card init successful");
    bootdrive = 0x80;
    insertdisk(0x80);
    hdcount = 1;
  }
#endif
  video_init();
  setup_ps2(PS2_DATA, PS2_IRQ);
  init8253();
#ifdef MEGA
  SPI.begin();
  SPI.setClockDivider(SPI_CLOCK_SPIRAM);
#else
  SPI1.begin();
  SPI1.setMOSI(21);
  SPI1.setMISO(5);
  SPI1.setSCK(20);
  SPI1.setClockDivider(SPI_CLOCK_SPIRAM);
#endif
  for (i = 0; i < 8; i++) {
    if (SPI_RAM_pins[i] > 0) {
      pinMode(SPI_RAM_pins[i], OUTPUT);
      digitalWrite(SPI_RAM_pins[i], HIGH);
    }
  }
  net_init();
}

void loop() {
  uint8_t i8253chan;
  reset86();
  while (1) {
    exec86(2000);
#ifdef USE_TFT
    updatescreen();
#endif
    //doirq(0);
    //exec86(3000);
    //pollkb();
    //ps2poll();
    net_loop();
    /*for (i8253chan=0; i8253chan<3; i8253chan++) {
        if (i8253.active[i8253chan]) {
            if (i8253.counter[i8253chan] < 10) i8253.counter[i8253chan] = i8253.chandata[i8253chan];
            i8253.counter[i8253chan] -= 10;
          }
      }*/
  }
}
